import os
import csv
import types
import sys
from time import time
from django.conf import settings
from django.db import connection, transaction
#from arches.app.models.entity import Entity
from arches.app.models.entity import Entity
from pprint import pprint
import collections

class Row(object):
    def __init__(self, *args):
        if len(args) == 0:
            self.resource_id = ''
            self.resourcetype = ''
            self.attributename = ''
            self.attributevalue = ''
            self.group_id = ''
        elif isinstance(args[0], list):
            self.resource_id = args[0][0].strip()
            self.resourcetype = args[0][1].strip()
            self.attributename = args[0][2].strip()
            self.attributevalue = args[0][3].strip()
            self.group_id = args[0][4].strip()

    def __repr__(self):
        return ('"%s, %s, %s, %s"') % (self.resource_id, self. resourcetype, self. attributename, self.attributevalue)

class Group(object):
    def __init__(self, *args):
        if len(args) == 0:
            self.resource_id = ''
            self.group_id = ''
            self.rows = []
        elif isinstance(args[0], list):
            self.resource_id = args[0][0].strip()
            self.group_id = args[0][4].strip()
            self.rows = []   

class Resource(object):
    def __init__(self, *args):
        if len(args) == 0:
            self.resource_id = ''
            self.entitytypeid = ''
            self.groups = []
            self.nongroups = []
        elif isinstance(args[0], list):
            self.entitytypeid = args[0][1].strip()
            self.resource_id = args[0][0].strip()
            self.nongroups = []  
            self.groups = []
    def appendrow(self, row, group_id=None):
        if group_id != None:
            for group in self.groups:
                if group.group_id == group_id:
                    group.rows.append(row)
        else:
           self.nongroups.append(row)

    def __str__(self):
        return '{0},{1}'.format(self.resource_id, self.entitytypeid)

class ResourceRelation(object):
    def __init__(self, *args):
        if len(args) == 0:
            self.resourceid1 = ''
            self.xref_entity_id1 = ''
            self.xref_value = ''
            self.resourceid2 = ''
            self.xref_entity_id2 = ''
            self.legacyid = ''
            self.relation_type = ''
            self.start_date = ''
            self.end_date = ''
    def __str__(self):
        return '{0},{1}'.format(self.start_date, self.end_date)


class DataLoader(object): 

    def load(self, filepath, resource_relation_entitytypeids=None, truncate=True):
        start = time()
        resource_info = open(filepath, 'rb')
        rows = [line.split("|") for line in resource_info]
        resourceList = []
        resource_id = ''
        group_id = ''

        if truncate:
            cursor = connection.cursor()
            cursor.execute("""
                TRUNCATE data.entities CASCADE;
            """ )

        for row in rows:
            if rows.index(row) != 0:
                if (settings.LIMIT_ENTITY_TYPES_TO_LOAD == None or row[1].strip() in settings.LIMIT_ENTITY_TYPES_TO_LOAD):
                    if row[0].strip() != resource_id:
                        resource = Resource(row)
                        resourceList.append(resource)
                        resource_id = row[0].strip()
                    
                    if row[4].strip() != '-1' and row[4].strip() != group_id:
                        resource.groups.append(Group(row))
                        group_id = row[4].strip()

                    if row[4].strip() == group_id:
                        resource.appendrow(Row(row), group_id=group_id)
                        #resourceList[len(resourceList)-1].groups[len(resourceList[len(resourceList)-1].groups)-1].rows.append(Row(row))

                    if row[4].strip() == '-1':
                        resource.appendrow(Row(row))
                        #resourceList[len(resourceList)-1].nongroups.append(Row(row))

        elapsed = (time() - start)
        print 'time to parse csv = %s' % (elapsed)
        return self.resource_list_to_entities(resourceList, resource_relation_entitytypeids, cursor)
    
    def resource_list_to_entities(self, resourceList, resource_relation_entitytypeids, cursor):
        start = time()
        ret = {'successfully_saved':0, 'successfully_indexed':0, 'failed_to_save':[], 'failed_to_index':[]}
        schema = None
        current_entitiy_type = None
        resource_relations = []



        for resource in resourceList:
            masterGraph = None
            entityData = []
            if current_entitiy_type != resource.entitytypeid:
                schema = Entity.get_mapping_schema(resource.entitytypeid)


            master_graph = self.build_master_graph(resource, schema)
            self.update_related_resources(master_graph, resourceList)



            master_graph.save(username=settings.ETL_USERNAME)
            resource.entityid = master_graph.entityid
            resource.legacyid = self.get_legacy_id(master_graph)




            # # Gather legacyids for all entities in resource_info.
            # if resource_relation_entitytypeids: 

            #     def traverse_down(entity, dictionary):
            #         for child in entity.relatedentities:
            #             if child.value != '':
            #                 dictionary[child.entitytypeid] = child.value
            #             traverse_down(child, dictionary)
            #         return dictionary

            #    for item in master_graph.find_entities_by_type_id('EXTERNAL XREF.E42'):
            #         relation = ResourceRelation()
            #         relation.resourceid1 = master_graph.entityid
            #         relation.legacyid = master_graph.entitytypeid.split('.')[0] + ':' + item.value
            #         resource_relations.append(relation)

            #     # If an entity in resource_info has an Arches Resource.E1 entity, grab it's value and the rest of the xref branch.
            #     for item in master_graph.find_entities_by_type_id('ARCHES RESOURCE.E1'):
            #         xref_attributes = {}
            #         relation = ResourceRelation()
            #         relation.resourceid1 = master_graph.entityid
            #         relation.xref_entity_id1 = item.entityid
            #         relation.xref_value = item.value
            #         xref_dic = {}
            #         if len(item.relatedentities) > 0:
            #             xref_attributes = traverse_down(item, xref_dic)
            #             # parse xref branch attributes and add to relation object
            #             if 'ARCHES RESOURCE CROSS-REFERENCE RELATIONSHIP TYPE.E55' in xref_attributes:
            #                 relation.relation_type = xref_attributes['ARCHES RESOURCE CROSS-REFERENCE RELATIONSHIP TYPE.E55']
            #             if 'DATE RESOURCE ASSOCIATION STARTED.E50' in xref_attributes:
            #                 relation.start_date = xref_attributes['DATE RESOURCE ASSOCIATION STARTED.E50']
            #             if 'DATE RESOURCE ASSOCIATION ENDED.E50' in xref_attributes:
            #                 relation.end_date = xref_attributes['DATE RESOURCE ASSOCIATION ENDED.E50']
            #         resource_relations.append(relation)

            ret['successfully_saved'] += 1
            master_graph.index()
            ret['successfully_indexed'] += 1

        # Compare resourceRelation objects and cross populate xref_entity_id's and resourceid's if resource.legacyid == related_resource.xref_value
        if resource_relation_entitytypeids and len(resource_relations) > 0:
            self.save_related_resources(resource_relations, resource_relation_entitytypeids, cursor)

        elapsed = (time() - start)
        print 'total time to etl = %s' % (elapsed)
        print 'average time per entity = %s' % (elapsed/len(resourceList))
        print ret
        return ret

    def build_master_graph(self, resource, schema):
        master_graph = None
        entity_data = []
        for row in resource.nongroups:
            entity = Entity()
            entity.create_from_mapping(row.resourcetype, schema[row.attributename]['steps'], row.attributename, row.attributevalue)
            entity_data.append(entity)

        if len(entity_data) > 0:
            master_graph = entity_data[0]
            for mapping in entity_data[1:]:
                master_graph.merge(mapping)

        for group in resource.groups:
            entity_data2 = []
            for row in group.rows:
                entity = Entity()
                entity.create_from_mapping(row.resourcetype, schema[row.attributename]['steps'], row.attributename, row.attributevalue)
                entity_data2.append(entity)  

            mapping_graph = entity_data2[0]
            for mapping in entity_data2[1:]:
                mapping_graph.merge(mapping)

            if master_graph == None:
                master_graph = mapping_graph
            else:
                node_type_to_merge_at = schema[row.attributename]['mergenodeid']
                master_graph.merge_at(mapping_graph, node_type_to_merge_at)

        return master_graph

    def get_legacy_id(self, entity):
        for item in entity.find_entities_by_type_id('EXTERNAL XREF.E42'):
            return item.value

    def update_related_resources(self, master_graph, resourceList):
        for item in master_graph.find_entities_by_type_id('ARCHES RESOURCE.E1'):
            for resource in resourceList:
                try:
                    if item.value == resource.legacyid:
                        item.value = resource.entityid
                except:pass

    def save_related_resources(self, resource_relations, resource_relation_entitytypeids, cursor):
        for resource in resource_relations:
            for resource2 in resource_relations:
                if resource2.legacyid and (resource.xref_value == resource2.legacyid):
                    resource.resourceid2 = resource2.resourceid1
                    resource.xref_entity_id2 = resource2.xref_entity_id1

            if resource.resourceid2:
                if resource.xref_entity_id2 == '':
                    resource.xref_entity_id2 = self.generate_uuid(cursor)

                    resource_entity = Entity().get(resource.resourceid2)
                    
                    mapping_nodes = collections.OrderedDict([
                        ('ARCHES RESOURCE.E1', resource.resourceid1),
                        ('ARCHES RESOURCE CROSS-REFERENCE RELATIONSHIP TYPE.E55', resource.relation_type),
                        ('DATE RESOURCE ASSOCIATION STARTED.E50', resource.start_date),
                        ('DATE RESOURCE ASSOCIATION ENDED.E50', resource.end_date)
                        ])

                    if resource_entity.entitytypeid in resource_relation_entitytypeids:
                        entities_to_merge = []
                        for entity_type_id, entity_value in mapping_nodes.iteritems():
                            entity = Entity() 
                            mapping_steps = self.get_mapping_steps(resource_entity, entity_type_id, cursor)
                            entity.create_from_mapping(resource_entity.entitytypeid, mapping_steps, entity_type_id, entity_value)
                            entities_to_merge.append(entity)

                        if len(entities_to_merge) > 0:
                            base_entity = entities_to_merge[0]
                            for entity in entities_to_merge[1:]:
                                base_entity.merge(entity)
                                
                            resource_entity.merge(base_entity)
                        
                            try:
                                resource_entity.save()
                                print 'successfully_saved'
                            except:
                                print 'failed to save' 

                #pprint(JSONSerializer().serializeToPython(resource_entity, ensure_ascii=True, indent=4))

                entity = Entity()
                related_resource = entity.get(resource.resourceid1)
                for item in related_resource.find_entities_by_type_id('ARCHES RESOURCE.E1'):
                    if item.value == resource.xref_value:
                        item.value = resource.resourceid2
                        related_resource.save()
                    else:
                        print '''ERROR: '%s' is not a valid resource legacyid''' %(item.value)

    def generate_uuid(self, cursor):
        sql = """
            SELECT uuid_generate_v1mc()
        """
        cursor.execute(sql)
        return cursor.fetchone()[0]

    def get_mapping_steps(self, resource_entity, leaf_entity_type_id, cursor):
        sql = """
            SELECT mappingid FROM ontology.mappings WHERE entitytypeidfrom = '%s' AND entitytypeidto = '%s'
        """%(resource_entity.entitytypeid, leaf_entity_type_id)
        cursor.execute(sql)
        return resource_entity._get_mappings(cursor.fetchone()[0])

    def findmergenode(self, graph):
        """
        Logic to try and determine the proper node to merge multiple instances of the same types of data. Like address info.
        Currently unused.  

        """

        rank = None
        highestnodewithvalue = None
        entitieswithvalues = []
        def findentitieswithvalues(entity):
            if entity.value != '':
                entitieswithvalues.append(entity)
        graph.traverse(findentitieswithvalues)

        if len(entitieswithvalues) == 0:
            raise Exception("Entity graph has no values")
        for entity in entitieswithvalues:
            if rank == None:
                rank = entity.get_rank()
            if entity.get_rank() <= rank:
                rank = entity.get_rank()
                highestnodewithvalue = entity

        rank = None
        highestforkingnode = None
        entitiesthatfork = []
        def findentitiesthatfork(entity):
            if len(entity.relatedentities) > 1:
                entitiesthatfork.append(entity)
        graph.traverse(findentitiesthatfork)

        for entity in entitiesthatfork:
            if rank == None:
                rank = entity.get_rank()
            if entity.get_rank() <= rank:
                rank = entity.get_rank()
                highestforkingnode = entity


        # LOGIC SECTION
        if highestforkingnode == None or highestnodewithvalue.get_rank() >= highestforkingnode.get_rank():
            if highestnodewithvalue.get_parent().get_rank() == 0:
                return highestnodewithvalue.get_parent()
            else:
                return highestnodewithvalue.get_parent().get_parent()

        if highestnodewithvalue.get_rank() < highestforkingnode.get_rank():
            if highestforkingnode.get_rank() == 0:
                return highestforkingnode
            else:
                return highestforkingnode.get_parent()

        return None

        #     groupNode =



        # 1.) Identify highest node with value;
        # 2.) Identify node at which the group has a common parent;

        # 3.) Take the node that is closer to the root
        #     a.) if that is that is the node with a value: split two parents up or root
        #     b.) else: split one parent up




from django.conf import settings
import csv
from django.db import connection
import unicodecsv
from datetime import datetime
from django.contrib.gis.geos import fromstr
from django.contrib.gis import geos
import decimal
import sys
import os
from arches.management.commands import utils
import time

class Row(object):
    def __init__(self, *args):
        if len(args) == 0:
            self.resource_id = ''
            self.resourcetype = ''
            self.attributename = ''
            self.attributevalue = ''
            self.group_id = ''
        elif isinstance(args[0], dict):
            details = args[0]
            self.resource_id = details['RESOURCEID'].strip()
            self.resourcetype = details['RESOURCETYPE'].strip()
            self.attributename = details['ATTRIBUTENAME'].strip()
            self.attributevalue = details['ATTRIBUTEVALUE'].strip().replace('\\n', '\n')
            self.group_id = details['GROUPID'].strip()

    def __repr__(self):
        return ('"%s, %s, %s, %s"') % (self.resource_id, self. resourcetype, self. attributename, self.attributevalue)

class Group(object):
    def __init__(self, *args):
        if len(args) == 0:
            self.resource_id = ''
            self.group_id = ''
            self.rows = []
        elif isinstance(args[0], dict):
            details = args[0]
            self.resource_id = details['RESOURCEID'].strip()
            self.group_id = details['GROUPID'].strip()
            self.rows = []   

class Resource(object):
    def __init__(self, *args):

        if len(args) == 0:
            self.resource_id = ''
            self.entitytypeid = ''
            self.groups = []
            self.nongroups = []

        elif isinstance(args[0], dict):
            details = args[0]
            self.entitytypeid = details['RESOURCETYPE'].strip()
            self.resource_id = details['RESOURCEID'].strip()
            self.nongroups = []  
            self.groups = []

    def appendrow(self, row, group_id=None):
        if group_id != None:
            for group in self.groups:
                if group.group_id == group_id:
                    group.rows.append(row)
        else:
           self.nongroups.append(row)

    def __str__(self):
        return '{0},{1}'.format(self.resource_id, self.entitytypeid)

class Validation_Errors(object):
    def __init__(self, *args):
        self.syntax_errors = []
        self.entitytype_errors = []
        self.businesstable_error = []
        self.domain_errors = []
        self.date_errors = []
        self.geometry_errors = []
        self.filepath_errors = []
        self.number_errors = []
        self.contiguousness_errors = []
        self.relations_errors = []

class Validator(object):
    def __init__(self, *args):
        self.resource_attributes = self.get_resource_attributes() # {resource1:{entitype:businesstable, entitytype:businesstable}, resource2:{entitype:businesstable, entitytype:businesstable}...}
        self.domain_values = self.get_domain_values()
        self.complete_resources = {}
        self.previous_resourceid = ''
        self.contiguous_errors = []
        self.current_group = {}
        self.group_errors = []
        self.unique_resourceids = set()
        self.errors = []

        self.validation_errors = Validation_Errors()

    def get_resource_attributes(self):
        cursor = connection.cursor()
        sql = """
            SELECT m.entitytypeidfrom, m.entitytypeidto, et.businesstablename FROM ontology.mappings m
            JOIN data.entity_types et ON et.entitytypeid = m.entitytypeidto
        """
        cursor.execute(sql)
        resource_types = []
        resource_dict = {}
        for val in cursor.fetchall():
            if val[0] in resource_dict:
                resource_dict[val[0]][val[1]] = val[2]
            else:
                resource_dict[val[0]] = {val[1] : val[2]}

        return resource_dict

    def get_domain_values(self):
        cursor = connection.cursor()
        sql = """
            SELECT DISTINCT(legacyoid) FROM concepts.concepts
        """
        cursor.execute(sql)
        domain_vals = []
        for val in cursor.fetchall():
            domain_vals.append(val[0])

        return domain_vals

    def validate_contiguousness(self, row, rownum):
        if row['RESOURCEID'] not in self.complete_resources:
            self.complete_resources[row['RESOURCEID']] = False
        else:
            if self.complete_resources[row['RESOURCEID']] != False and row['RESOURCEID'] not in self.contiguous_errors:
                self.append_error('ERROR ROW: {0} - {1} has noncontiguous attributes.'.format(rownum, row['RESOURCEID']), 'contiguousness_errors')
                self.contiguous_errors.append(row['RESOURCEID'])

        if self.previous_resourceid != row['RESOURCEID']:
            self.complete_resources[self.previous_resourceid] = True

        self.previous_resourceid = row['RESOURCEID']

    def validate_row_syntax(self, row, rownum):
        # property for row num, also value for missing property row['MISSING'] same for additional
        if 'MISSING' in row.values():
            self.append_error('row missing value', 'syntax_errors')

        if 'ADDITIONAL' in row.values():
            self.append_error('additional values present in row', 'syntax_errors')

    def validate_entitytype(self, row, rownum):
        if row['RESOURCETYPE'] in self.resource_attributes:
            if row['ATTRIBUTENAME'] in self.resource_attributes[row['RESOURCETYPE']]:
                pass
            else:
                self.append_error('{0} not a valid entity type for {1} resource.'.format(row['ATTRIBUTENAME'], row['RESOURCETYPE']), 'entitytype_errors')
        else:
            self.append_error('{0} is not a valid resource type.'.format(row['RESOURCETYPE']), 'entitytype_errors')


    def valdiate_attribute_value(self, row, rownum):
        entity_type = row['ATTRIBUTENAME']
        resource_type = row['RESOURCETYPE']
        business_table = self.get_businesstable(resource_type, entity_type)

        if business_table not in ['strings', 'dates', 'domains', 'files', 'geometries', 'numbers']:
            self.append_error('{0} is not a valid business table name.'.format(business_table), 'businesstable_error')
        else:
            if business_table == 'domains':
                self.validate_domains(row, rownum)

            elif business_table == 'dates':
                self.validate_dates(row, rownum)

            elif business_table == 'geometries':
                self.validate_geometries(row, rownum)

            # DO NOT DELETE: Uncomment this when we have acual file paths to test against.
            # elif business_table == 'files':
            #     self.validate_files(row, rownum)

            elif business_table == 'numbers':
                self.validate_numbers(row, rownum)


    def validate_domains(self, row, rownum):
        if row['ATTRIBUTEVALUE'] not in self.domain_values:
            self.append_error('ERROR ROW:{0} - {1} is not a valid domain value. Check authority document related to {2}'.format(rownum, row['ATTRIBUTEVALUE'], row['ATTRIBUTENAME']), 'domain_errors')

    def validate_dates(self, row, rownum):
        date_formats = settings.DATE_PARSING_FORMAT
        valid = False
        for format in date_formats:
            if valid == False: 
                try:
                    if datetime.strptime(row['ATTRIBUTEVALUE'], format):
                        valid = True
                except:
                    valid = False
        if valid == False:
            self.append_error('ERROR ROW: {0} - {1} is not a properly formatted date. Dates must be in {2}, or {3} format.'.format(rownum, row['ATTRIBUTEVALUE'], (',').join(date_formats[:-1]), date_formats[-1]), 'date_errors')

    def validate_geometries(self, row, rownum):
        try:
            geom = fromstr(row['ATTRIBUTEVALUE'])
            coord_limit = 1500
            bbox = geos.Polygon(settings.DATA_VALIDATION_BBOX)

            if geom.num_coords > coord_limit:
                self.append_error('ERROR ROW: {0} - {1} has too many coordinates ({2}), Please limit to less then {3} coordinates of 5 digits of precision or less.'.format(rownum, row['ATTRIBUTEVALUE'][:75] + '......', geom.num_coords, coord_limit), 'geometry_errors')

            # if fromstr(row['ATTRIBUTEVALUE']).valid == False:
            #     self.append_error('ERROR ROW: {0} - {1} is an invalid geometry.'.format(rownum, row['ATTRIBUTEVALUE']), 'geometry_errors')

            if bbox.contains(geom) == False:
                self.append_error('ERROR ROW: {0} - {1} does not fall within the bounding box of the selected coordinate system. Adjust your coordinates or your settings.DATA_EXTENT_VALIDATION property.'.format(rownum, row['ATTRIBUTEVALUE']), 'geometry_errors')
            
        except:
            self.append_error('ERROR ROW: {0} - {1} is not a properly formatted geometry.'.format(rownum, row['ATTRIBUTEVALUE']), 'geometry_errors')

    def validate_numbers(self, row, rownum):
        try:
            decimal.Decimal(row['ATTRIBUTEVALUE'])
        except:
            self.append_error('ERROR ROW: {0} - {1} is not a properly formatted number.'.format(rownum, row['ATTRIBUTEVALUE']), 'number_errors')

    def validate_files(self, row, rownum):
        """
        This method will have to be changed to include prefix for file directory.
        """
        if os.path.isfile(row['ATTRIBUTEVALUE']):
                pass
        else:
            self.append_error('ERROR ROW: {0} - {1} file either does not exist at location or is not readable.'.format(rownum, row['ATTRIBUTEVALUE']), 'filepath_errors')

    def validate_relations_file(self, arches_file):
        unique_relationids = set()
        relations_file = arches_file.replace('.arches', '.relations')
        with open(relations_file, 'rU') as f:
            fieldnames = ['RESOURCEID_FROM','RESOURCEID_TO','START_DATE','END_DATE','RELATION_TYPE','NOTES']
            rows = unicodecsv.DictReader(f, fieldnames=fieldnames, 
                encoding='utf-8-sig', delimiter='|', restkey='ADDITIONAL', restval='MISSING')
            rows.next()
            rownum = 2
            for row in rows:
                self.validate_row_syntax(row, rownum)
                unique_relationids.add(row['RESOURCEID_FROM'])
                unique_relationids.add(row['RESOURCEID_TO'])
                rownum += 1

        for resourceid in unique_relationids - self.unique_resourceids:
            self.append_error('"{0}" is present in {1} but not {2}. Both resourceids must be present in {3} in order to create a valid relation.'.format(resourceid, relations_file.split('/')[-1], arches_file.split('/')[-1], arches_file.split('/')[-1]), 'relations_errors')

    def get_businesstable(self, resource, attributename):
        try:
            return self.resource_attributes[resource][attributename]
        except:
            self.append_error('No business table for {0} in {1}. Check if attributename is valid and that business table name is populated for {2} in {3}.'.format(attributename, resource, attributename, resource), 'businesstable_error')

    def append_error(self, text, error_type):
        error_type_list = getattr(self.validation_errors, error_type)
        error_type_list.append(text)

class ArchesReader():

    def validate_file(self, arches_file, break_on_error=True):
        """
        Creates row dictionaries from a csv file

        """
        validator = Validator()
        with open(arches_file, 'rU') as f:
            fieldnames = ['RESOURCEID','RESOURCETYPE','ATTRIBUTENAME','ATTRIBUTEVALUE','GROUPID']
            rows = unicodecsv.DictReader(f, fieldnames=fieldnames, 
                encoding='utf-8-sig', delimiter='|', restkey='ADDITIONAL', restval='MISSING')
            rows.next() # skip header row
            rownum = 2
            start_time = time.time()
            for row in rows:
                validator.validate_row_syntax(row, rownum)
                validator.validate_entitytype(row, rownum)
                validator.valdiate_attribute_value(row, rownum)
                validator.validate_contiguousness(row, rownum)

                validator.unique_resourceids.add(row['RESOURCEID'])
                rownum += 1
        validator.validate_relations_file(arches_file)
        duration = time.time() - start_time

        sorted_errors = []
        attr_length = len(validator.validation_errors.__dict__)
        for attr, value in validator.validation_errors.__dict__.iteritems():
            if value != []:
                sorted_errors.extend(value)
                sorted_errors.append('\n\n\n\n')
        if len(sorted_errors) > 1:
            del sorted_errors[-1]

        print 'Validation of your Arches file took: {0} seconds.'.format(str(duration))
        if len(sorted_errors) > 0:
            utils.write_to_file(os.path.join(settings.PACKAGE_ROOT, 'logs', 'validation_errors.txt'), '\n'.join(sorted_errors))
            print "\n\nERROR: There were errors detected in your arches file."
            print "Please review the errors at %s, \ncorrect the errors and then rerun this script." % (os.path.join(settings.PACKAGE_ROOT, 'logs', 'validation_errors.txt'))
            if break_on_error:
                sys.exit(101)

    def load_file(self, arches_file):
        '''Reads an arches data file and creates resource graphs'''
        resource_list = []
        resource_id = ''
        group_id = ''
        resource = ''

        resource_info = csv.DictReader(open(arches_file, 'r'), delimiter='|')

        # Import package specific validation module.
        fully_qualified_modulename = settings.PACKAGE_VALIDATOR
        components = fully_qualified_modulename.split('.')
        classname = components[len(components)-1]
        modulename = ('.').join(components[0:len(components)])
        validation_module = __import__(modulename, globals(), locals(), [classname], -1)
        start_time = time.time()

        for row in resource_info:
            # print row
            
            group_val = row['GROUPID'].strip()
            resource_type_val = row['RESOURCETYPE'].strip()
            resource_id_val = row['RESOURCEID'].strip()

            if (settings.LIMIT_ENTITY_TYPES_TO_LOAD == None or resource_type_val in settings.LIMIT_ENTITY_TYPES_TO_LOAD):
                if resource_id_val != resource_id:
                    if resource != '':
                        validation_module.validate_resource(resource)
                    resource = Resource(row)
                    resource_list.append(resource)
                    resource_id = resource_id_val
                    group_id = ''
                
                if group_val != '-1' and group_val != group_id:  #create a new group of resouces
                    resource.groups.append(Group(row))
                    group_id = group_val

                if group_val == group_id:
                    resource.appendrow(Row(row), group_id=group_id)

                if group_val == '-1':
                    resource.appendrow(Row(row))

        validation_module.validate_resource(resource)

        sorted_errors = []
        attr_length = len(validation_module.validation_errors.__dict__)
        for attr, value in validation_module.validation_errors.__dict__.iteritems():
            if value != []:
                sorted_errors.extend(value)
                sorted_errors.append('\n\n\n\n')
        if len(sorted_errors) > 1:
            del sorted_errors[-1]
        duration = time.time() - start_time
        print 'Validation of your business data took: {0} seconds.'.format(str(duration))
        if len(sorted_errors) > 0:
            utils.write_to_file(os.path.join(settings.PACKAGE_ROOT, 'logs', 'validation_errors.txt'), '\n'.join(sorted_errors))
            print "\n\nERROR: There were errors detected in your business data."
            print "Please review the errors at %s, \ncorrect the errors and then rerun this script." % (os.path.join(settings.PACKAGE_ROOT, 'logs', 'validation_errors.txt'))
            break_on_error = True
            if break_on_error:
                sys.exit(101)

        return resource_list